//------------------------------------------------------------------------------
//
// PROJECT : Fruechtebrot Fractal Batch
//
// FILE    : main.cpp
//
// VERSION : 1.0
//
//---------------------------------------------------------------------------

#include <iostream>
#include <fstream>
#include <vector>
#include <list>

using namespace std;


#include <string.h>
#include <CRectT.h>
#include <CPixel24.h>
#include <CGraflib.h>
#include <CGraflibRaw.h>
#include <CStringTool.hpp>
#include <CComplex.h>
#include <CComplex.hpp>
#include <CDatabase3.h>
#include <math.h>

#include "SColorProfile.h"
#include "CThread.h"
#include "CSuperThread.hpp"
#include "CFractThread.h"
#include "CLogTable.h"
#include "SFractParams.h"
#include "CFraktBitmap.h"
#include "CFractBatch.h"
#include "CPicWriter.h"




// ---------------------------------------------------------------------------
//
// KLASSE        : CFractBatchThread
// METHODE       : CalcFraktalThread
//
//
// CGraflib* tga - Zeiger auf TGA-Grafikobjekt
// int Threads   - Anzahl der Threads
//
// ---------------------------------------------------------------------------

void CFractBatch::CalcFraktalThread(CGraflibTga* tga, CGraflib* raw, int Threads)
{
  CPicWriter PicWriter;
	CComplex<double> c, z;
	static CFractThread*  fthread = NULL;

	int FractWitdh  = tga->mWidth * 4;
  int FractHeight = tga->mHeight * 4;

	

  double StepX = (mFractParams.mFrect.right - mFractParams.mFrect.left) / FractWitdh;
	double StepY = StepX;

	PicWriter.InitPicWriter(tga);

  c.imag = mFractParams.mFrect.top;

  if (fthread == NULL)
	{
		fthread = new CFractThread[Threads];
    mSuperThread.InitThreads(fthread, NULL, Threads);
	} 

	for (int t = 0; t < Threads; t++)
	{
		fthread[t].mFractData = new int[FractWitdh];
	}

	int tlines = FractHeight / Threads;
	int g = tlines / 50;


	cout << "InitThreads OK" << endl;

  for (int y = 0; y < tlines; y++)
  {
	  //cout << "y=" << y << endl;

		for (int t = 0; t < Threads; t++)
		{
			fthread[t].mC.real = mFractParams.mFrect.left;
			fthread[t].mC.imag = c.imag;
			fthread[t].mStepX  = StepX;
			fthread[t].mIter   = mFractParams.mIter;
			fthread[t].mWidth  = FractWitdh;
			c.imag += StepY;
		}

		//cout << "Starting Threads.." << endl;
		mSuperThread.StartThreads(fthread, Threads);
		//cout << "ok, waiting for end.." << endl;
		mSuperThread.WaitEndCondition();
		//		cout << "ok, making bitmaps" << endl;

		for (int t = 0; t < Threads; t++)
		{
			CPixel24* BitmapData = new CPixel24[FractWitdh];
			mFractParams.CreateFractBitmap(fthread[t].mFractData, BitmapData, 1);
			//cout << " create ok" << t << endl;

			PicWriter.AddBitmap(BitmapData);
			PicWriter.Sync();

      /*
			if (raw != NULL)
			{
				raw->WriteLine(fthread[t].mFractData);
			}
      */
			//cout << " write ok" << t << endl;
		}
		

 		


		if ((y % g) == 0)
		{
			cout << '.';
			cout.flush();
		}
	}
	//delete[] BitmapData;

	for (int t = 0; t < Threads; t++)
	{
		delete[] fthread[t].mFractData;
	}
	PicWriter.Finish();
 
	cout << endl;
	
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CFractBatch
// METHODE       : RenderTga
//
// string TgaFilename - Name der Tga-Datei
// int Threads        - Anzahl der Threads
//
// ---------------------------------------------------------------------------

void CFractBatch::RenderTga(const string& TgaFilename, int Threads)
{
	char* RawFilename;
	CGraflibTga  tga;
	CGraflibRaw* raw = NULL;

	tga.mWidth  = mFractParams.mXres;
	tga.mHeight = mFractParams.mYres;
	tga.mBits = 24;

	mFractParams.mXres *= 4;
  mFractParams.mYres *= 4;
	
	if (mSaveRaw)
	{
	  CFileIO fio;
		raw = new CGraflibRaw;
		raw->mWidth  = tga.mWidth;
		raw->mHeight = tga.mHeight;
		raw->mBits   = 32;
		
		RawFilename = fio.ReplaceApp(TgaFilename.c_str(), ".raw");
		raw->OpenWriteLine(RawFilename); 
	}	
	
	if (tga.OpenWriteLine(TgaFilename.c_str()))
	{
		CalcFraktalThread(&tga, raw, Threads);
		//tga.1Line();
	}
	
	if (mSaveRaw)
	{
		raw->CloseWriteLine();
		delete RawFilename;
		delete raw;
	}	
}

// ---------------------------------------------------------------------------
//
// KLASSE        : CFractBatch
// METHODE       : ParseBatchFile
//
// Parsen der Batch-Datei und berechnen der Fraktale 
// 
// string BatchFile - Name der Batch-Text-Datei
// int Width - Breite der Grafik
// int Height - Hoehe der Grafik
// int Iter   - Anzahl der Iterationen
// int Thread - Anzahl der Threads 
// 
//
// ---------------------------------------------------------------------------

void CFractBatch::ParseBatchFile(string BatchFile, int Width, int Height, int Iter,
  int Threads)
{
	CStringTool   StringTool;
	CDatabase3    db3;
	list<CRecord> Records;


	string BasePath = "";
	string FilePath = BatchFile;
	string TgaFilename;
	int FileCount = 0;
	int TotalFileCount = 0;



	if (db3.Read(&Records, BasePath, FilePath, '\t'))
	{
		for (list<CRecord>::iterator rc  = Records.begin();
			                           rc != Records.end();
			                           rc++)
		{
			if ((*rc)[0] == "NAME")
			{
				TotalFileCount += 1;
			}
		}

		for (list<CRecord>::iterator rc  = Records.begin();
			                           rc != Records.end();
			                           rc++)
		{

			if ((*rc)[0] == "NAME")
			{
				TgaFilename = (*rc)[1];
			}
			else
			if ((*rc)[0] == "RECT")
			{
				mFractParams.mFrect.left   = StringTool.StringTo<double>((*rc)[1]);
				mFractParams.mFrect.top    = StringTool.StringTo<double>((*rc)[2]);
				mFractParams.mFrect.right  = StringTool.StringTo<double>((*rc)[3]);
				mFractParams.mFrect.bottom = StringTool.StringTo<double>((*rc)[4]);
			}
			else
			if ((*rc)[0] == "COLOR")
			{
				mFractParams.mCol.mRed   = StringTool.StringTo<float>((*rc)[1]);
				mFractParams.mCol.mGreen = StringTool.StringTo<float>((*rc)[2]);
				mFractParams.mCol.mBlue  = StringTool.StringTo<float>((*rc)[3]);
			}
			else
			if ((*rc)[0] == "LIMIT1")
			{
				mFractParams.mLimit1 = StringTool.StringTo<int>((*rc)[1]);
			}
			else
			if ((*rc)[0] == "POTENZ")
			{
				mFractParams.mPotenz = StringTool.StringTo<float>((*rc)[1]);
			}
			else
			if ((*rc)[0] == "MINVAL")
			{
				mFractParams.mMinVal = StringTool.StringTo<int>((*rc)[1]);
			}
			else
			if ((*rc)[0] == "ENDNAME")
			{
			  FileCount += 1;
				cout << "Render " << FileCount
				     << " of " << TotalFileCount
				     << ": " << TgaFilename << endl;
				mFractParams.mXres = Width;
				mFractParams.mYres = Height;
				mFractParams.mIter = Iter;	

			  RenderTga(TgaFilename, Threads);
			}
		}
	}
}
