#include <iostream>
#include <fstream>
#include <math.h>
#include <string.h>

using namespace std;

#include <CRectT.h>

#include "CFraktBitmap.h"

#define KP 256

void CFraktBitmap::CreateColorTables(float fr, float fg, float fb, unsigned char* RedTable, unsigned char* GreenTable, unsigned char* BlueTable)
{
  if (fr <= 0) fr = 0.01;
  if (fg <= 0) fg = 0.01;
  if (fb <= 0) fb = 0.01;

	for (int i = 0; i < 256*256; i++)
	{
		int ir = int(fr / 10 * i) % 256;
		int ig = int(fg / 10 * i) % 256;
		int ib = int(fb / 10 * i) % 256;

		float kr =  (float(ir) / (KP)) * (M_PI * 2);
		float kg  = (float(ig) / (KP)) * (M_PI * 2);					
		float kb  = (float(ib) / (KP)) * (M_PI * 2);

		float sr = (sin(kr) * 127) + 128;
		float sg = (sin(kg) * 127) + 128;
		float sb = (sin(kb) * 127) + 128;

		RedTable[ir]   = (int) sr;
		GreenTable[ig] = (int) sg;		
    BlueTable[ib]  = (int) sb;
	}
}




#define COLOR_METHOD 1

//---------------------------------------------------------------------------
//
// Klasse:	CFraktal
// Methode:	CreateFractBitmap
//
//
//---------------------------------------------------------------------------

void CFraktBitmap::CreateFractBitmap(const int* FractData, CPixel24* Bitmap, int Lines)
{
	//cout << "CFraktal::CreateFractBitmap2 START" << endl;

	CPixel24* BitmapPtr = Bitmap;
	

	if (COLOR_METHOD != 0)
	{	
		if (!mInitTables)
		{
			mGreenTable = new unsigned char[258];
			mRedTable   = new unsigned char[258];
			mBlueTable  = new unsigned char[258];
			CreateColorTables(mCol.mRed, mCol.mGreen, mCol.mBlue, mRedTable, mGreenTable, mBlueTable);

			if (mLimit1 > 0)
			{
				mLogtable.Init(1, mLimit1, 0,255, mPotenz, true);
			}
			mInitTables = true;
		}
	}

	const int* FractDataPtr = FractData;

	for (int y = 0; y < Lines; y++)
	{
		for (int x = 0; x < mXres; x++)
		{
			int i = *FractDataPtr - mMinVal;

			if (i >= 0)
			{
				switch (COLOR_METHOD)
				{
					case 0:
					{
						int i = *FractDataPtr;
						int Blau  = (int) (mCol.mBlue * i);
						int Gruen = (int) (mCol.mGreen * i);
						int Rot   = (int) (mCol.mRed * i);
					  BitmapPtr->Set(Rot & 0xFF, Gruen & 0xFF, Blau & 0xFF);
					}
					break;

					case 1:
					{
						int i = *FractDataPtr - mMinVal;
						if (i < mLimit1)
						{

							int c = 255 - mLogtable.GetLogMap(i);

							//float c = (float(mLimit1 - i) / mLimit1) * 255.0;

							BitmapPtr->Set(c, c, c);
						}
						else
						{
				      int ir = int(mCol.mRed * i) % 256;
				      int ig = int(mCol.mGreen * i) % 256;
				      int ib = int(mCol.mBlue * i) % 256;

				      int sr = mRedTable[ir];
				      int sg = mGreenTable[ig];
					    int sb = mBlueTable[ib];

							BitmapPtr->Set(sr, sg, sb);
						}
					}
					break;
				}
			}
			else
			{
				BitmapPtr->Set(0,0,0);
			}
			BitmapPtr++;
			FractDataPtr++;
		}
	}


	//cout << "CFraktBitmap::CreateFractBitmap OK" << endl;
}
