//------------------------------------------------------------------------------
//
// PROJECT : Portable Thread
//
// FILE    : CSuperThread.cpp
//
// VERSION : 1.0
//
// AUTOR   : Martin Steen
//
//
//
//------------------------------------------------------------------------------
//
// Inhalt: Implementation der Klasse CSuperThread
//
//------------------------------------------------------------------------------
// Revisionsgeschichte:
//
// 07.09.2007 erstellt - Martin Steen
// 18.11.2008 modifiziert - Martin Steen
//
//---------------------------------------------------------------------------

#if defined _WIN32_
#include <windows.h>
#endif

#if defined __unix__
#include <pthread.h>
#endif

#include <iostream>

using namespace std;


#include "CThread.h"

//---------------------------------------------------------------------------
//
// Klasse:    CSuperThread
// Methode:		CSuperThread
//
//
//---------------------------------------------------------------------------

CSuperThread::CSuperThread()
{
	mInit = false;
}


//---------------------------------------------------------------------------
//
// Klasse:    CSuperThread
// Methode:		LockSuperMutex
//
//
//---------------------------------------------------------------------------

void CSuperThread::LockSuperMutex()
{
	#if defined __unix__
	pthread_mutex_lock(&mSuperMutex);
	#else
	#if defined __WIN32__
	EnterCriticalSection(&mCriticalSection);
	#else
	Weder Unix noch Windows?
	#endif
	#endif
}

//---------------------------------------------------------------------------
//
// Klasse:    CSuperThread
// Methode:		InitMutex
//
//
//---------------------------------------------------------------------------

void CSuperThread::InitMutex()
{
	#ifdef __unix__
	pthread_mutex_init(&mSuperMutex, NULL);
	pthread_cond_init(&mEndCondition, NULL);
	pthread_mutex_init(&mEndMutex, NULL);
	#endif

	#ifdef __WIN32__
	mEndCondition  = CreateEvent(NULL, FALSE, FALSE, NULL);
	InitializeCriticalSection(&mCriticalSection);
	//cout << "InitializeCriticalSection ok" << endl;
	#endif

}

//---------------------------------------------------------------------------
//
// Klasse:    CSuperThread
// Methode:		WaitEndCondition
//
//
//---------------------------------------------------------------------------

void CSuperThread::WaitEndCondition()
{
	#ifdef __unix__
	pthread_mutex_unlock(&mSuperMutex);
	pthread_cond_wait(&mEndCondition, &mEndMutex);
	#endif

	#ifdef __WIN32__
	LeaveCriticalSection(&mCriticalSection);
	WaitForSingleObject(mEndCondition, INFINITE);
	#endif
}

//---------------------------------------------------------------------------
//
// Klasse:    CSuperThread
// Methode:		SetEndCondition
//
//
//---------------------------------------------------------------------------

void CSuperThread::SetEndCondition()
{
	#ifdef __unix__
	pthread_mutex_lock(&mEndMutex);
	pthread_cond_signal(&mEndCondition);
	pthread_mutex_unlock(&mEndMutex);
	#endif

	#ifdef __WIN32__
	SetEvent(mEndCondition);
	#endif
}


