 //------------------------------------------------------------------------------
//
// PROJECT : Portable Thread
//
// FILE    : CThread.cpp
//
// VERSION : 1.0
//
// AUTOR   : Martin Steen
//           Imagon GmbH
//
//
//
//------------------------------------------------------------------------------
//
// Inhalt: Implementation der Klasse CThread
//
//------------------------------------------------------------------------------
// Revisionsgeschichte:
//
// 07.09.2007 erstellt - Martin Steen
//
//---------------------------------------------------------------------------

#if defined _WIN32_
#include <windows.h>
#endif

#if defined __unix__
#include <pthread.h>
#endif


//#include "OsInclude.h"
#include <iostream>

using namespace std;


#include "CThread.h"

//---------------------------------------------------------------------------
//
// Klasse:    static
// Methode:		StandardThreadFunktion
//
// Standard-Serviceroutine fuer den Thread. Diese Funktion ruft die
// virtuelle Methode "ThreadServiceRoutine" der Thread-Klasse auf.
//
//---------------------------------------------------------------------------

static void* StandardThreadFunktion(void* tp)
{
	CThread* thread = (CThread*) tp;
	while (1)
	{
		thread->WaitStart();
		thread->ThreadServiceRoutine();
		thread->DecThreadCount();
	}
	return NULL;
}

//---------------------------------------------------------------------------
//
// Klasse:    CThread
// Methode:		ExitThread
//
//---------------------------------------------------------------------------

void CThread::ExitThread()
{
	#if defined __unix__
	pthread_exit(NULL);
	#else
	::ExitThread(0);
	#endif
}


//---------------------------------------------------------------------------
//
// klasse:    CThread
// Methode:		Create
//
// Erzeugen eines Thread mit Standard-Parametern
//
//---------------------------------------------------------------------------

void CThread::Create(CSuperThread* st, void* StartVar)
{
	mSuperThread = st;
	Create(StandardThreadFunktion, StartVar);
}

//---------------------------------------------------------------------------
//
// Klasse:    CThread
// Methode:		Create
//
// Erzeugen eines Thread
//
//---------------------------------------------------------------------------

void CThread::Create(void* (*ServiceRoutine)(void*), void* StartVar)
{
	mStartVar = StartVar;
  #ifdef __unix__

	pthread_cond_init(&mStartCondition, NULL);
	pthread_mutex_init(&mStartMutex, NULL);

  if (pthread_create(&mThreadHandle, NULL,  StandardThreadFunktion, this) != 0)
  {
    cout << "***** pthread_create failed!" << endl;
  }
  usleep(25);
  //cout << "ok" << endl;
	#endif

	#ifdef  __WIN32__
	//cout << "CThread::Create" << endl;

	mStartCondition  = CreateEvent(NULL, FALSE, FALSE, NULL);
	mThreadHandle    = CreateThread(NULL, 0, (LPTHREAD_START_ROUTINE) StandardThreadFunktion, this, 0, &mThreadId);
	Sleep(25);

	//cout << "CThread::Create ok" << endl;

	#endif

}



//---------------------------------------------------------------------------
//
// Klasse:    CThread
// Methode:		DecThreadCount
//
//
//---------------------------------------------------------------------------

void CThread::DecThreadCount()
{
  //cout << "DecThreadCount" << endl;
	#ifdef __unix__
	if (mSuperThread != NULL)
	{
		pthread_mutex_lock(&mSuperThread->mSuperMutex);
		if (mSuperThread->mThreadCount > 0)
		{
      
			mSuperThread->mThreadCount--;
			if (mSuperThread->mThreadCount == 0)
			{
				mSuperThread->SetEndCondition();
			}
		}
		pthread_mutex_unlock(&mSuperThread->mSuperMutex);
	}
	#endif

	#ifdef  __WIN32__
	EnterCriticalSection(&mSuperThread->mCriticalSection);
	if (mSuperThread->mThreadCount > 0)
	{
		mSuperThread->mThreadCount--;
		if (mSuperThread->mThreadCount == 0)
		{
			mSuperThread->SetEndCondition();
		}
	}
	LeaveCriticalSection(&mSuperThread->mCriticalSection);
	#endif

}




//---------------------------------------------------------------------------
//
// Klasse:    CThread
// Methode:		WaitStart
//
// Warten auf das Start-Signal
//
//---------------------------------------------------------------------------

void CThread::WaitStart()
{
	#if defined __unix__
	pthread_cond_wait(&mStartCondition, &mStartMutex);
	#elif defined __WIN32__
	WaitForSingleObject(mStartCondition, INFINITE);
	#endif
}



//---------------------------------------------------------------------------
//
// Klasse:    CThread
// Methode:		Start
//
// Setzen des Start-Signals
//
//---------------------------------------------------------------------------

void CThread::Start()
{
	#if defined __unix__
	pthread_mutex_lock (&mStartMutex);
	pthread_cond_signal(&mStartCondition);
	pthread_mutex_unlock(&mStartMutex);
	#elif defined __WIN32__
	SetEvent(mStartCondition);
	#else
	Weder Unix noch Windows?
	#endif
}


