

#ifndef CTHREAD_H
#define CTHREAD_H

#if defined __WIN32__
#include <windows.h>
#elif defined __unix__
#include <pthread.h>
#include <unistd.h>
#endif

class CThread;

class CSuperThread
{
	public:

		CSuperThread(void);



		template <typename TThread>
		void InitThreads(TThread* threads, void** vars, int tcount);

		template <typename TThread>
		void StartThreads(TThread* threads, int tcount);

		void WaitEndCondition(void);

		friend class CThread;

	protected:

		void InitMutex(void);
		void SetThreadCount(int count);
		void SetEndCondition(void);
		void LockSuperMutex(void);


		bool            mInit;
		volatile int    mThreadCount;

		#ifdef __unix__
		pthread_mutex_t mSuperMutex;
		pthread_mutex_t mEndMutex;
		pthread_cond_t  mEndCondition;
		#endif

		#ifdef __WIN32__
		HANDLE           mEndCondition;
		CRITICAL_SECTION mCriticalSection;
		#endif

};


class CThread
{
	public:

		void Create(CSuperThread* SuperThread, void* StartVar);
		void Create(void* (*ServiceRoutine)(void*), void* StartVar);

		void ExitThread(void);
		void ThreadSleep(int msec);
		void Join(void);

		void DecThreadCount(void);

		void Start(void);
		void WaitStart(void);

		virtual void ThreadServiceRoutine(void) = 0;

	protected:

		CSuperThread* mSuperThread;   // Zeiger auf Supervisor-Thread
		void*         mStartVar;

	#ifdef __WIN32__
		DWORD  mThreadId;
		HANDLE mThreadHandle;
		HANDLE mStartCondition;
	#endif

	#ifdef __unix__
		pthread_t       mThreadHandle;
		pthread_mutex_t mStartMutex;
		pthread_cond_t  mStartCondition;
	#endif

};

#endif
