//------------------------------------------------------------------------------
//
// PROJECT : Imagon
//
// FILE    : CGraflibRaw.cpp
//
// VERSION : 1.0
//
// AUTOR   : Martin Steen
//           Imagon GmbH
//
//
//
//------------------------------------------------------------------------------
//
// Inhalt: Implementation der Klasse CGraflibRaw
//
//------------------------------------------------------------------------------
// Revisionsgeschichte:
//
// 5/2003 erstellt - Martin Steen
//
//---------------------------------------------------------------------------

//#include <windows.h>

#include <iostream>
#include <fstream>
#include <vector>
#include <string.h>

using namespace std;

#include <CFileIO.h>
#include <SFileReadInfo.h>
#include <CGraflibRaw.h>
#include <CStringTool.h>

//---------------------------------------------------------------------------
//
// Klasse:		SRawHeader
// Methode:		Read
//
// Parameter:
//
// Return:
//
//---------------------------------------------------------------------------


bool SRawHeader::Write(const char*  RifFilename)
{
	fstream fs(RifFilename, ios::out);

	if (fs.good())
	{
		fs << "IDD=RAW1" << endl;
		fs << "WID=" << mWidth << endl;;
		fs << "HGT=" << mHeight << endl;
		fs << "BPP=" << mBits << endl;

		fs.close();

		return true;
	}
	return false;
}

//---------------------------------------------------------------------------
//
// Klasse:		SRawHeader
// Methode:		Read
//
// Parameter:
//
// Return:
//
//---------------------------------------------------------------------------


bool SRawHeader::Read(const char*  Filename)
{
	fstream fs(Filename, ios::in);

	if (fs.good())
	{
		string CfgLine;

		//cout << "SRawHeader::Read START" << endl;

		do
		{
			fs >> CfgLine;

			if (!fs.eof())
			{
				if (CfgLine.size() > 4)
				{
					//cout << "CfgLine=" << CfgLine << endl;

					CStringTool st;
					vector<string> res;

					st.SplitString(CfgLine, &res, '=');

					if (res.size() == 2)
					{
						if (res[0] == "IDD")
						{

						}
						else
						if (res[0] == "WID")
						{
							mWidth = st.StringTo<int>(res[1]);
						}
						else
						if (res[0] == "HGT")
						{
							mHeight = st.StringTo<int>(res[1]);
						}
						else
						if (res[0] == "BPP")
						{
							mBits = st.StringTo<int>(res[1]);
						}
						else
						{
							cout << "***** unknown keyword in SRawHeader::Read:" << res[0] << endl;
						}
					}
					//cout << "ok" << endl;
				}
			}
		}
		while (!fs.eof() > 0);

		fs.close();

		return true;
	}
	return false;
}


//---------------------------------------------------------------------------
//
// Klasse:		CGraflibRaw
// Methode:		OpenTiffRead
//
// Parameter:
//
// Return:
//
//---------------------------------------------------------------------------

bool CGraflibRaw::OpenReadLine(const char* Filename)
{
	CFileIO fio;

	char* RifName = fio.ReplaceApp(Filename, ".rif");

	if (fio.OpenFileRead(RifName, ios::binary))
	{
		SRawHeader rh;

		rh.Read(RifName);

		mWidth  = rh.mWidth;
		mHeight = rh.mHeight;
		mBits   = rh.mBits;
		delete RifName;

		mFs = new fstream(Filename, ios::in | ios::binary);

		if (!mFs->good())
		{
			throw(CGraflibException(EGLIB_OPENREAD, Filename));
			return false;
		}
		return true;
	}
	else
  {
		throw(CGraflibException(EGLIB_OPENREAD, RifName));
	}
	return false;
}

//---------------------------------------------------------------------------
//
// Klasse:		CGraflibRaw
// Methode:		CloseTiffRead
//
// Parameter:
//
// Return:
//
//---------------------------------------------------------------------------

void CGraflibRaw::CloseReadLine()
{
	mFs->close();
	delete mFs;
	mFs = NULL;
}


//---------------------------------------------------------------------------
//
// Klasse:		CGraflibRaw
// Methode:		CloseTiffWrite
//
// Parameter:
//
// Return:
//
//---------------------------------------------------------------------------

void CGraflibRaw::CloseWriteLine()
{
	mFs->close();
	delete mFs;
	mFs = NULL;
}


//---------------------------------------------------------------------------
//
// Klasse:		CGraflibRaw
// Methode:		ReadScanLine
//
// Parameter:
//
// Return:
//
//---------------------------------------------------------------------------


void CGraflibRaw::ReadLine(void* LineBuffer)
{
	mFs->read((char*) LineBuffer, mWidth * (mBits/8));
}

//---------------------------------------------------------------------------
//
// Klasse:		CGraflibRaw
// Methode:		Read
//
// Parameter:
//
// Return:
//
//---------------------------------------------------------------------------

bool CGraflibRaw::Read(const char* Filename, SFileReadInfo* fri)
{
	//cout << "CGraflibRaw::Read START " << Filename << endl;

	if (OpenReadLine(Filename))
	{
		int LineSize = (mWidth * mBits) / 8;
		mData = new char[LineSize * mHeight];

		mFs->read((char*) mData, LineSize * mHeight);
		CloseReadLine();

		return true;
  }
  else
  {
		throw(CGraflibException(EGLIB_OPENREAD, Filename));
	}
  return false;
}

//---------------------------------------------------------------------------
//
// Klasse:    CGraflibRaw
// Methode:		Write
//
//
// Return:
//
//---------------------------------------------------------------------------

int CGraflibRaw::Write(const char*  Filename, bool bComp, CFileIO* fio)
{
	//cout << "CGraflibRaw::Write START " << Filename << endl;
	//ShowInfo();
	if (OpenWriteLine(Filename))
	{
		int LineSize = (mWidth * mBits) / 8;
		mFs->write((char*) mData, LineSize * mHeight);
		CloseWriteLine();
	}
	else
	{
		throw(CGraflibException(EGLIB_OPENWRITE, Filename));
	}
	return 0;
}

//---------------------------------------------------------------------------
//
// Klasse:    CGraflibRaw
// Methode:		OpenWrite
//
//
// Return:
//
//---------------------------------------------------------------------------

bool CGraflibRaw::OpenWriteLine(const char*  Filename)
{
	mFs = new fstream(Filename, ios::out | ios::binary);

	if (mFs->good())
	{
		SRawHeader rh;
		CFileIO    fio;

		char* RifName = fio.ReplaceApp(Filename, ".rif");

		strncpy(rh.mIdent, "RAW1", 4);
		rh.mWidth  = mWidth;
		rh.mHeight = mHeight;
		rh.mBits   = mBits;

		rh.Write(RifName);

		delete RifName;

		return true;
	}
	else
	{
		throw(CGraflibException(EGLIB_OPENWRITE, Filename));
	}
	return false;
}

//---------------------------------------------------------------------------
//
// Klasse:    CGraflibRaw
// Methode:		WriteScaleline
//
//
// Return:
//
//---------------------------------------------------------------------------

void CGraflibRaw::WriteLine(void* Scanline)
{
	mFs->write((char*) Scanline, mWidth * (mBits / 8));
}

